function B=anisotropic_averaging(A,num_epochs,long_epoch,first)

% Filters the matrix taking into account the similarity between the time series of 
% the pixels.
% The measure of similarity between time series is actually the cross correlation 
% between the spectrums only at the selected frequencies (0 to 0.4 Hz).
% The parameters are:
% A: The matrix containing the time series which are being analyzed.
% total_time: The total time of the acquisition, is needed to calculate the sample frequency.

dim = size(A)

lim=num_epochs*5;

FAux=abs(fft(A,[],3));
for i=1:dim(1)
   for j=1:dim(2)
      FA(i,j,:)=FAux(i,j,2:lim)/norm(squeeze(FAux(i,j,2:lim)));
   end
end
clear FAux;


% First filters the corners of the grid.

% Upper left corner.
target = squeeze(FA(1,1,:));
w(:,1)=squeeze(A(1,1,:));
destiny = squeeze(FA(1,2,:)); % Right neighbour.
total_weight = similarity(target,destiny,long_epoch,num_epochs,first);
w(:,2)=total_weight.*squeeze(A(1,2,:));
destiny = squeeze(FA(2,2,:)); % Right bottom neighbour.
aux=similarity(target,destiny,long_epoch,num_epochs,first);
total_weight=total_weight+aux;
w(:,3)=aux.*squeeze(A(2,2,:));
destiny = squeeze(FA(2,1,:)); % Bottom neighbour.
aux=similarity(target,destiny,long_epoch,num_epochs,first);
total_weight=total_weight+aux; 
w(:,4)=aux.*squeeze(A(2,1,:));
% Updates the "pixel" value by the mean of the weighted vectors.
B(1,1,:)=sum(w,2)/(1+total_weight); 

% Upper right corner.
target = squeeze(FA(1,dim(2),:));
w(:,1)=squeeze(A(1,dim(2),:));
destiny = squeeze(FA(1,dim(2)-1,:)); % Left neighbour.
total_weight=similarity(target,destiny,long_epoch,num_epochs,first); 
w(:,2)=total_weight.*squeeze(A(1,dim(2)-1,:));
destiny = squeeze(FA(2,dim(2)-1,:)); % Left bottom neighbour.
aux=similarity(target,destiny,long_epoch,num_epochs,first);
total_weight=total_weight+aux; 
w(:,3)=aux.*squeeze(A(2,dim(2)-1,:));
destiny = squeeze(FA(2,dim(2),:)); % Bottom neighbour.
aux=similarity(target,destiny,long_epoch,num_epochs,first);
total_weight=total_weight+aux; 
w(:,4)=aux.*squeeze(A(2,dim(2),:));
% Update the "pixel" value
B(1,dim(2),:)=sum(w,2)/(1+total_weight);

% Down left corner.
target = squeeze(FA(dim(1),1,:));
w(:,1)=squeeze(A(dim(1),1,:));
destiny = squeeze(FA(dim(1),2,:)); % Right neighbour.
total_weight=similarity(target,destiny,long_epoch,num_epochs,first); 
w(:,2)=total_weight.*squeeze(A(dim(1),2,:));
destiny = squeeze(FA(dim(1)-1,2,:)); % Right up neighbour.
aux=similarity(target,destiny,long_epoch,num_epochs,first);
total_weight=total_weight+aux; 
w(:,3)=aux.*squeeze(A(dim(1)-1,2,:));
destiny = squeeze(FA(dim(1)-1,1,:)); % Up neighbour.
aux=similarity(target,destiny,long_epoch,num_epochs,first);
total_weight=total_weight+aux; 
w(:,4)=aux.*squeeze(A(dim(1)-1,1,:));
% Update the "pixel" value
B(dim(1),1,:)=sum(w,2)/(1+total_weight); 

% Down right corner.
target = squeeze(FA(dim(1),dim(2),:));
w(:,1)=squeeze(A(dim(1),dim(2),:));
destiny = squeeze(FA(dim(1),dim(2)-1,:)); % Left neighbour.
total_weight=similarity(target,destiny,long_epoch,num_epochs,first);
w(:,2)=total_weight.*squeeze(A(dim(1),dim(2)-1,:));
destiny = squeeze(FA(dim(1)-1,dim(2)-1,:)); % Left up neighbour.
aux=similarity(target,destiny,long_epoch,num_epochs,first);
total_weight=total_weight+aux; 
w(:,3)=aux.*squeeze(A(dim(1)-1,dim(2)-1,:));
destiny = squeeze(FA(dim(1)-1,dim(2),:)); % Up neighbour.
aux=similarity(target,destiny,long_epoch,num_epochs,first);
total_weight=total_weight+aux; 
w(:,4)=aux.*squeeze(A(dim(1)-1,dim(2),:));
% Update the "pixel" value
B(dim(1),dim(2),:)=sum(w,2)/(1+total_weight); 


% Second, filter the first and last row/column (s)

% First row
for j=2:(dim(2)-1)
   target = squeeze(FA(1,j,:));
   w(:,1)=squeeze(A(1,j,:));
   destiny = squeeze(FA(1,j-1,:)); % Left neighbour.
   total_weight=similarity(target,destiny,long_epoch,num_epochs,first);
   w(:,2)=total_weight.*squeeze(A(1,j-1,:));
   destiny = squeeze(FA(1,j+1,:)); % Right neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,3)=aux.*squeeze(A(1,j+1,:));
   destiny = squeeze(FA(2,j,:)); 	 % Bottom neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,4)=aux.*squeeze(A(2,j,:));
   destiny = squeeze(FA(2,j-1,:)); % Left bottom neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,5)=aux.*squeeze(A(2,j-1,:));
   destiny = squeeze(FA(2,j+1,:)); % Right bottom neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,6)=aux.*squeeze(A(2,j+1,:));
   B(1,j,:)=sum(w,2)/(1+total_weight); 
end

% Last row
for j=2:(dim(2)-1)
   target = squeeze(FA(dim(1),j,:));
   w(:,1)=squeeze(A(dim(1),j,:));
   destiny = squeeze(FA(dim(1),j-1,:)); 	% Left neighbour.
   total_weight=similarity(target,destiny,long_epoch,num_epochs,first);
   w(:,2)=total_weight.*squeeze(A(dim(1),j-1,:));
   destiny = squeeze(FA(dim(1),j+1,:)); 	% Right neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,3)=aux.*squeeze(A(dim(1),j+1,:));
   destiny = squeeze(FA(dim(1)-1,j,:)); 	% Up neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,4)=aux.*squeeze(A(dim(1)-1,j,:));
   destiny = squeeze(FA(dim(1)-1,j-1,:)); 	% Left up neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,5)=aux.*squeeze(A(dim(1)-1,j-1,:));
   destiny = squeeze(FA(dim(1)-1,j+1,:)); 	% Right up neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,6)=aux.*squeeze(A(dim(1)-1,j+1,:));
   B(dim(1),j,:)=sum(w,2)/(1+total_weight); 
end

% First column
for i=2:(dim(1)-1)
   target = squeeze(FA(i,1,:));
   w(:,1)=squeeze(A(i,1,:));
   destiny = squeeze(FA(i,2,:)); 		% Right neighbour.
   total_weight=similarity(target,destiny,long_epoch,num_epochs,first); 
   w(:,2)=total_weight.*squeeze(A(i,2,:));
   destiny = squeeze(FA(i-1,1,:)); 	% Up neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,3)=aux.*squeeze(A(i-1,1,:));
   destiny = squeeze(FA(i+1,1,:)); 	% Down neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,4)=aux.*squeeze(A(i+1,1,:));
   destiny = squeeze(FA(i-1,2,:)); 	% Right up neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,5)=aux.*squeeze(A(i-1,2,:));
   destiny = squeeze(FA(i+1,2,:)); % Right down neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,6)=aux.*squeeze(A(i+1,2,:));
   B(i,1,:)=sum(w,2)/(1+total_weight); 
end

% Last column
for i=2:(dim(1)-1)
   target = squeeze(FA(i,dim(2),:));
   w(:,1)=squeeze(A(i,dim(2),:));
   destiny = squeeze(FA(i,dim(2)-1,:)); 	% Left neighbour.
   total_weight=similarity(target,destiny,long_epoch,num_epochs,first);    
	w(:,2)=total_weight.*squeeze(A(i,dim(2)-1,:));
   destiny = squeeze(FA(i-1,dim(2),:)); 	% Up neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,3)=aux.*squeeze(A(i-1,dim(2),:));
   destiny = squeeze(FA(i+1,dim(2),:)); 	% Down neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,4)=aux.*squeeze(A(i+1,dim(2),:));
   destiny = squeeze(FA(i-1,dim(2)-1,:)); 	% Left up neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,5)=aux.*squeeze(A(i-1,dim(2)-1,:));
   destiny = squeeze(FA(i+1,dim(2)-1,:));  % Left down neighbour.
   aux=similarity(target,destiny,long_epoch,num_epochs,first);
   total_weight=total_weight+aux; 
   w(:,6)=aux.*squeeze(A(i+1,dim(2)-1,:));
   B(i,dim(2),:)=sum(w,2)/(1+total_weight); 
end

% Regular pixels, namely those who are not in the border of the grid.

for i=2:(dim(1)-1)
   for j=2:(dim(2)-1)
      target = squeeze(FA(i,j,:));
      w(:,1)=squeeze(A(i,j,:));
      destiny = squeeze(FA(i,j-1,:)); 	% Left neighbour.
      total_weight=similarity(target,destiny,long_epoch,num_epochs,first); 
      w(:,2)=total_weight.*squeeze(A(i,j-1,:));
      destiny = squeeze(FA(i-1,j,:)); 	% Up neighbour.
      aux=similarity(target,destiny,long_epoch,num_epochs,first);
      total_weight=total_weight+aux; 
      w(:,3)=aux.*squeeze(A(i-1,j,:));
      destiny = squeeze(FA(i+1,j,:)); 	% Down neighbour.
      aux=similarity(target,destiny,long_epoch,num_epochs,first);
      total_weight=total_weight+aux; 
      w(:,4)=aux.*squeeze(A(i+1,j,:));
      destiny = squeeze(FA(i-1,j-1,:)); % Left up neighbour.
      aux=similarity(target,destiny,long_epoch,num_epochs,first);
      total_weight=total_weight+aux; 
      w(:,5)=aux.*squeeze(A(i-1,j-1,:));
      destiny = squeeze(FA(i+1,j-1,:)); % Left down neighbour.
      aux=similarity(target,destiny,long_epoch,num_epochs,first);
      total_weight=total_weight+aux; 
      w(:,6)=aux.*squeeze(A(i+1,j-1,:));
      destiny = squeeze(FA(i,j+1,:)); 	% Right neighbour.
      aux=similarity(target,destiny,long_epoch,num_epochs,first);
      total_weight=total_weight+aux; 
      w(:,7)=aux.*squeeze(A(i,j+1,:));
      destiny = squeeze(FA(i-1,j+1,:)); % Right up neighbour.
      aux=similarity(target,destiny,long_epoch,num_epochs,first);
      total_weight=total_weight+aux; 
      w(:,8)=aux.*squeeze(A(i-1,j+1,:));
      destiny = squeeze(FA(i+1,j+1,:)); % Right down neighbour.
      aux=similarity(target,destiny,long_epoch,num_epochs,first);
      total_weight=total_weight+aux; 
      w(:,9)=aux.*squeeze(A(i+1,j+1,:));
      B(i,j,:)=sum(w,2)/(1+total_weight); 
   end
end

